@@ -12,6 +12,7 @@
 #endif
 
 #include <stdio.h>
+#include <stdlib.h>	/*  getenv */
 #include <time.h>
 #include <locale.h>
 
@@ -37,6 +38,12 @@
 #define GPS_ON_ICON "gps_location"
 #define GPS_OFF_ICON "gps_not_connected"
 
+#define OSSO_GDL_NAME    "gps_data_logger"
+#define OSSO_GDL_SERVICE "org.maemo."OSSO_GDL_NAME
+#define OSSO_GDL_OBJECT  "/org/maemo/"OSSO_GDL_NAME
+#define OSSO_GDL_IFACE   "org.maemo."OSSO_GDL_NAME
+
+
 HD_DEFINE_PLUGIN_MODULE (GpsDataLoggerWidget, gps_data_logger_widget, HD_TYPE_HOME_PLUGIN_ITEM)
 
 static void
@@ -1129,6 +1136,73 @@ on_button_expose(GtkWidget *widget, GdkEventExpose *expose_event, gpointer data)
     return FALSE;
 }
 
+/* Callback for hardware D-BUS events */
+void hw_event_handler(osso_hw_state_t *state, gpointer data)
+{
+    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
+    gboolean should_stop_recording = FALSE;
+#ifdef DEBUG
+    if (self->logfile) {
+        g_fprintf(self->logfile, "%s -- enter\n", __func__);
+        g_fprintf(self->logfile, "  hw event received - ");
+        fflush(self->logfile);
+    }
+#endif
+
+    if (state->shutdown_ind)
+    {
+        hildon_banner_show_information(NULL, NULL, 
+                                       "Shutdown event!");
+        should_stop_recording = TRUE;
+#ifdef DEBUG
+        if (self->logfile) {
+            g_fprintf(self->logfile, "  Shutdown event!\n");
+            fflush(self->logfile);
+        }
+#endif
+    }
+    if (state->save_unsaved_data_ind)
+    {
+        hildon_banner_show_information(NULL, NULL, 
+                                       "Must save unsaved data event!");
+        should_stop_recording = TRUE;
+#ifdef DEBUG
+        if (self->logfile) {
+            g_fprintf(self->logfile, "  Must save unsaved data event!\n");
+            fflush(self->logfile);
+        }
+#endif
+    }
+/*    if (state->memory_low_ind)
+    {
+        hildon_banner_show_information(NULL, NULL, 
+                                       "Memory low event!");
+#ifdef DEBUG
+        if (self->logfile) {
+            g_fprintf(self->logfile, "  Memory low event!\n");
+            fflush(self->logfile);
+        }
+#endif
+    }
+    if (state->system_inactivity_ind)
+    {
+        hildon_banner_show_information(NULL, NULL, 
+                                       "Minimize application inactivity event!");
+#ifdef DEBUG
+        if (self->logfile) {
+            g_fprintf(self->logfile, "  Minimize application inactivity event!\n");
+            fflush(self->logfile);
+        }
+#endif
+    } */
+    /* shall we stop recording and flush the log? */
+    if ((self->app_state != APP_STATE_IDLE) && should_stop_recording) {
+        /* let's prettend that the user pressed stop */
+        handle_button_press(self->button_stop, self);
+    }
+}
+
+
 static void
 gps_data_logger_widget_init (GpsDataLoggerWidget * self)
 {
@@ -1136,7 +1210,24 @@ gps_data_logger_widget_init (GpsDataLoggerWidget * self)
     GtkWidget *status_text;
     GdkScreen *screen;
     GdkColormap *colormap;
+    osso_return_t result;
+    GString *version = NULL ;
+    GString *tmp = NULL;
 
+    version = g_string_new(getenv("OSSO_PRODUCT_RELEASE_VERSION"));
+#ifndef __arm__
+    g_print("SW version: %s\n", version->str);
+#endif
+    self->osso_context = osso_initialize(OSSO_GDL_SERVICE, "0.0.1", TRUE, NULL);
+    /* Check that initialization was ok */
+    if (self->osso_context == NULL) {
+#ifndef __arm__
+        g_print("init : osso_initialize failed.\n");
+#endif
+        hildon_banner_show_information(NULL,
+			NULL,
+			"osso_initialize failed");
+    }
     self->file = NULL;
 #ifdef DEBUG
     self->logfile = NULL;
@@ -1229,24 +1320,27 @@ gps_data_logger_widget_init (GpsDataLoggerWidget * self)
     g_signal_connect (self, "show-settings", G_CALLBACK (on_settings_dialog_show),
 		    self);
 
-    /* button signals */
-
-    g_signal_connect(G_OBJECT(self->button_rec), "button-release-event",
-		    G_CALLBACK(on_button_release), self);
-    g_signal_connect(G_OBJECT(self->button_rec), "button-press-event",
-		    G_CALLBACK(on_button_press), self);
-    g_signal_connect(G_OBJECT(self->button_stop), "button-release-event",
-		    G_CALLBACK(on_button_release), self);
-    g_signal_connect(G_OBJECT(self->button_stop), "button-press-event",
-		    G_CALLBACK(on_button_press), self);
-    /* On the first release of Fremantle all the button events are
-     * going to the background, so let's register those too
-     * ToDo: figure out how to check for the updated sw version
-     *       and not register button signals for the background there */
-    g_signal_connect(G_OBJECT(self->background), "button-release-event",
-                    G_CALLBACK(on_button_release), self);
-    g_signal_connect(G_OBJECT(self->background), "button-press-event",
-                    G_CALLBACK(on_button_press), self);
+    tmp = g_string_new("1.");
+    g_string_truncate(version, 2);
+    if (g_string_equal(version, tmp)) {
+        /* On the first release of Fremantle all the button events are
+         * going to the background, so let's register those too */
+        g_signal_connect(G_OBJECT(self->background), "button-release-event",
+                         G_CALLBACK(on_button_release), self);
+        g_signal_connect(G_OBJECT(self->background), "button-press-event",
+                         G_CALLBACK(on_button_press), self);
+    } else {
+        /* button signals */
+        g_signal_connect(G_OBJECT(self->button_rec), "button-release-event",
+		         G_CALLBACK(on_button_release), self);
+        g_signal_connect(G_OBJECT(self->button_rec), "button-press-event",
+		         G_CALLBACK(on_button_press), self);
+        g_signal_connect(G_OBJECT(self->button_stop), "button-release-event",
+		         G_CALLBACK(on_button_release), self);
+        g_signal_connect(G_OBJECT(self->button_stop), "button-press-event",
+		         G_CALLBACK(on_button_press), self);
+    }
+    
     g_signal_connect(G_OBJECT(self->background), "leave-notify-event",
                     G_CALLBACK(on_widget_leave), self);
     /* expose events */
@@ -1264,6 +1358,21 @@ gps_data_logger_widget_init (GpsDataLoggerWidget * self)
 #ifdef DEBUG
     debuglog_start(self);
 #endif
+    if (self->osso_context) {
+        /* Add handler for hardware D-BUS messages */
+        result = osso_hw_set_event_cb( self->osso_context,
+            NULL, hw_event_handler, (gpointer)self );
+
+        if (result != OSSO_OK)
+        {
+#ifndef __arm__
+            g_print("Error setting HW state callback (%d)\n", result);
+#endif
+        }
+    }
+    g_string_free(version, TRUE);
+    g_string_free(tmp, TRUE);
+
 }
 
 static void
@@ -1285,6 +1394,11 @@ gps_data_logger_widget_finalize (GObject *object)
     g_free(self->settings);
     free_icons(self);
     free_theme_pixbufs(self);
+    /* Deinitialize OSSO */
+    if (self->osso_context) {
+        osso_deinitialize(self->osso_context);
+        self->osso_context = NULL;
+    }
 		    
     /* Call the base class's implementation: */
     G_OBJECT_CLASS(gps_data_logger_widget_parent_class)->finalize (object);
